//
// powerstation.gs
//
//  Copyright (C) 2003 Auran Developments Pty Ltd
//  All Rights Reserved.
//

include "GenericIndustry.gs"


//
// The scriptlet for the power station is rather basic.  All it does is unload coal from slow moving
// hoppers and activate/deactivate the particle effects as required.  As well as that, the 
// consumption rate of the coal consumer process can be changed through the implemented 
// PropertyObject methods.
//
// The coal_consumer process runs itself as long as there is coal to consumer (which is dependent
// on coal being dumped as the coal input queue is initially empty in default config).
//
class PowerStation isclass GenericIndustry
{
  ProductQueue coalInQueue;       // Coal input queue.
  bool scriptletEnabled = true;   // Enabled/disabled indicator flag of this scriptlet.

  // Track if they only supply some of the coal that was requested in the waybill.
  int coalWBRemain = 0;


  //
  // GenericIndustry methods
  //

  //
  // Displays statistics on current levels of various queues in pop-up browser window.
  //
  thread void ViewDetails(void)
  {
    if (!info)
    {
      info = Constructors.NewBrowser();
      
      info.LoadHTMLString(HTMLWindow.GetCompleteIndustryViewDetailsHTMLCode(me, scriptletEnabled));
	  info.SetWindowRect(100, 80, 500, 545);
    }
  }

  //
  // Returns false to indicate that this no loading commands will be handled by this industry.
  //
  bool HandleTrainLoadCommand(Train train)
  {
    return false;
  }

  //
  // Returns true to indicate that the trigger allows loading/unloading while a vehicle is moving.
  //
  bool TriggerSupportsMovingLoad(Vehicle vehicle, string triggerName)
  {
    if (itc.IsTrainCommand(vehicle.GetMyTrain(), Industry.UNLOAD_COMMAND))
      return true;

    return false;
  }

  //
  // Unloads coal from a hopper into the coal pile (input queue).
  //
  void PerformMovingLoad(Vehicle vehicle, string triggerName)
  {
    float speed = vehicle.GetVelocity();

    if (speed > -5.0f and speed < 5.0f)
    {
      int roomAvailable = coalInQueue.GetQueueSpace();
      LoadingReport report = CreateUnloadingReport(coalInQueue, roomAvailable);
      vehicle.UnloadProduct(report);

      // Ensure we are tracking this if we are waiting for a way bill to complete in its entirety
      if (coalWBRemain > 0)
        coalWBRemain = coalWBRemain - report.amount;
    }

  }


  //
  // PowerStation methods
  //

  //
  // Main thread that monitors the coal consumer process and enables/disables particle effects
  // for unloading.
  //
  thread void CoalMain(void)
  {
    Message msg;

    wait()
    {
      // Power Rule has sent message indicating power is running, so update enabled flag and ensure
      // that coal_consumer process is enabled.
      on "Scriptlet-Enabled", "1":
      {
        if (!scriptletEnabled)
        {
          scriptletEnabled = true;
          SetProcessEnabled("coal_consumer", true);
        }
        continue;
      }
      // Power Rule has sent message indicating power is not running, so update enabled flag and
      // and ensure that coal_consumer process is disabled.
      on "Scriptlet-Enabled", "0":
      {
        if (scriptletEnabled)
        {
          scriptletEnabled = false;
          SetProcessEnabled("coal_consumer", false);
        }
        continue;
      }

      // enable particle effects when coal consuming process starts
      on "Process-Start", "coal_consumer":
      {
        SendMessage(me, "pfx", "+0+1+2+3+4+5+6");
        continue;
      }
      // disable particle effects when coal consuming process has stopped
      on "Process-Stop", "coal_consumer":
      {
        SendMessage(me, "pfx", "-0-1-2-3-4-5-6");
        continue;
      }
    }
  }

  //
  // Game object initialization method, called by Trainz when the industry is first created.  Is
  // responsible for initializing data members and starting threads to run the industry.
  //
  public void Init(void)
  {
    inherited();

    coalInQueue = GetQueue("coal_in");
    
    AddAssetToIndustryProductInfo("coal", "coal_in", "coal_consumer", true);

    CoalMain();
  }


  //
  // Industry methods
  //
  
  //
  // Gets the requirements of this power station (coal drop off request) for use in a waybill.
  //
  public Requirement[] GetRequirements(void)
  {
    Requirement[] ret = new Requirement[0];
    
    if (coalInQueue.GetQueueCount() < 162900 or coalWBRemain > 0) // 3 coal hoppers
    {
      ResourceRequirement req = new ResourceRequirement();
      
      req.resource = coalInQueue.GetProductFilter().GetProducts()[0];
      
      // This is how many we have asked for. Wait till it is fulfilled, 
      // if we are not already waiting for a waybill to be completed.
      req.amount = 651600;        // 12 coal hoppers
      if (coalInQueue.GetQueueCount() < 162900 and coalWBRemain == 0)
        coalWBRemain = 651600;

      req.dst = me;
      req.dstQueue = coalInQueue;

      ret[ret.size()] = req;
    }
    
    return ret;
  }

  //
  // Provides information so the coal drop track destination can appear in this industry's
  // sub-menu for the 'Drive To >' command menu.
  //
  public void AppendDriverDestinations(string[] destNames, string[] destTracks)
  {
    destNames[destNames.size()] = "Coal Drop off";
    destTracks[destTracks.size()] = "in_track";
  }


};

